<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Captcha
 *
 * @copyright   (C) 2011 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

require_once 'User.php';
require_once 'Achievements.php';

class PlgContentUsersync extends JPlugin
{

	protected $autoloadLanguage = true;

	private $realm_slug;
	private $guild_slug;
	private $region;
	private $locale;
	private $token;
	private $link_wow_user;
	private $sync_achievements;
	private $achievements_list;
	private $sync_user_detail;
	private $timezone;

	public function __construct(&$subject, $config = array()){

		parent::__construct($subject, $config);

		$this->realm_slug = $this->params->get('realm_slug');
		$this->guild_slug = $this->params->get('guild_slug');
		$this->region = $this->params->get('region');
		$this->locale = $this->params->get('locale');
		$this->link_wow_user = $this->params->get('link_wow_user');
		$this->sync_achievements = $this->params->get('sync_achievements');
		$this->achievements_list = $this->params->get('achievements_list');
		$this->sync_user_detail = $this->params->get('sync_user_detail');
		$this->timezone = $this->params->get('timezone');
	}

	function isPluginBattlenetActive(){
		return !(empty(JPluginHelper::getPlugin('system', 'battlenet')));
	}

	function allParameterSet(){

		$error = 0;

		if($this->realm_slug === null || $this->realm_slug == '')
			$error++;
		if($this->guild_slug === null || $this->guild_slug == '')
			$error++;
		if($this->region === null || $this->region == '')
			$error++;
		if($this->locale === null || $this->locale == '')
			$error++;
		if($this->link_wow_user === null || $this->link_wow_user == '')
			$error++;
		if($this->sync_achievements === null || $this->sync_achievements == '')
			$error++;
		if($this->achievements_list === null || $this->achievements_list == '')
			$error++;
		if($this->sync_user_detail === null || $this->sync_user_detail == '')
			$error++;
		if($this->timezone === null || $this->timezone == '')
			$error++;

		return $error == 0;
	}

	function getToken(){
		$battlenet_config = JPluginHelper::getPlugin('system', 'battlenet');
		$token = json_decode($battlenet_config->params)->client_token;
		return $token;
	}

	function requestGetGuildRoster(){

		$curl = curl_init();
		$response = new stdClass();

		curl_setopt_array($curl, array(
			CURLOPT_URL => 'https://'.$this->region.'.api.blizzard.com/data/wow/guild/'.$this->realm_slug.'/'.$this->guild_slug.'/roster?namespace=profile-'.$this->region.'&access_token='.$this->token.'&locale='.$this->locale,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => '',
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => 'GET',
		));

		if(curl_exec($curl) === false){
			$response->error = curl_error($curl);
		}else{
			$response = json_decode(curl_exec($curl));
		}

		curl_close($curl);

		/**
		 * stdClass Object
		(
		[access_token] => USxBBl7P6336Kk1cwVhlI9B8d3pc9RR4vT
		[token_type] => bearer
		[expires_in] => 86399
		[sub] => aa0c7df71dfa4e9194998c6797fa61f4
		)
		 */
		return $response;
	}

	public function requestGetCharacterAchievements($user){
		$curl = curl_init();

		$response = new stdClass();

		curl_setopt_array($curl, array(
			CURLOPT_URL => 'https://'.$this->region.'.api.blizzard.com/profile/wow/character/'.$user->get('realm').'/'.$user->get('user_slug').'/achievements?namespace=profile-'.$this->region.'&access_token='.$this->token.'&locale='.$this->locale,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => '',
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => 'GET',
		));

		if(curl_exec($curl) === false){
			$response->error = curl_error($curl);
		}else{
			$response = json_decode(curl_exec($curl));
		}

		curl_close($curl);
		return $response;
	}

	public function requestGetCharacterDetail($user){
		$curl = curl_init();

		$response = new stdClass();

		curl_setopt_array($curl, array(
			CURLOPT_URL => 'https://'.$this->region.'.api.blizzard.com/profile/wow/character/'.$user->get('realm').'/'.$user->get('user_slug').'?namespace=profile-'.$this->region.'&access_token='.$this->token.'&locale='.$this->locale,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => '',
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => 'GET',
		));

		if(curl_exec($curl) === false){
			$response->error = curl_error($curl);
		}else{
			$response = json_decode(curl_exec($curl));
		}

		curl_close($curl);
		return $response;
	}

	public function cronUpdate($current_part = null, $parts = null){

		$load_all = false;

		if($this->isPluginBattlenetActive() && $this->allParameterSet()){

			$this->token = $this->getToken();
			$guild_roster = $this->requestGetGuildRoster();

			if(!isset($guild_roster->error)){

				$members = $guild_roster->members;

				if($current_part !== null && $parts !== null && $current_part <= $parts){
					$load_all = true;
					$member_count = count($members);
					$load_to = $current_part * ceil($member_count/$parts) - 1;
					$load_from = 1 * ($load_to + 1 - ceil($member_count/$parts));
					$load_to = ($current_part == $parts) ? $member_count : $load_to;
				}

				foreach($members as $key => $member){

					if($load_all && $key < $load_from)
						continue;

					if($load_all && $key > $load_to)
						break;

					$character = $member->character;

					$user = new User();
					$user->set('user_name', $character->name);
					$user->set('level', $character->level);
					$user->set('rank', $member->rank);
					$user->set('battle_id', $character->id);
					$user->set('realm', $character->realm->slug);
					$user->set('user_slug', $character->key->href);

					if($this->link_wow_user == 1){
						$user->addJoomlaUserId();
					}

					if(($user->get('user_id') != 0 || $load_all) && $this->sync_user_detail){

						$user_detail = $this->requestGetCharacterDetail($user);

						if(isset($user_detail->last_login_timestamp)){
							$d = DateTime::createFromFormat('U.v', number_format($user_detail->last_login_timestamp/1000, 3, '.', ''));
							$la_time = new DateTimeZone($this->timezone);
							$d->setTimezone($la_time);
							$last_login_time =  $d->format('Y-m-d H:i:s') . "\n";

							$user->set('gender', $user_detail->gender->name);
							$user->set('race', $user_detail->race->name);
							$user->set('class', $user_detail->character_class->name);
							$user->set('specialization', $user_detail->active_spec->name);
							$user->set('achievement_points', $user_detail->achievement_points);
							$user->set('last_login_timestamp', $last_login_time);
							$user->set('equipped_item_level', $user_detail->equipped_item_level);
							$user->set('active_title', (isset($user_detail->active_title->name)) ? $user_detail->active_title->name : '');
						}
					}

					$user->save();

					if(
						$this->sync_achievements != 0 &&
						($user->get('user_id') != 0 || $load_all)
					){
						$achievement = new Achievements($user);
						$achievements = $this->requestGetCharacterAchievements($user);
						$achievement_list = explode('|', $this->achievements_list);

						if(isset($achievements->achievements)){
							foreach($achievements->achievements as $key => $value){
								if(
									($this->sync_achievements == 1) || // All
									($this->sync_achievements == 2 && in_array($value->id, $achievement_list)) // Selected
								){

									if(!isset($value->criteria->is_completed)){
										$completed = 0;
									}else{
										$completed = ($value->criteria->is_completed) ? 1 : 0;
									}

									$achievement->addAchievement(
										$value->id,
										$value->achievement->name,
										$completed
									);
								}
							}

							$achievement->save();
						}
					}
				}
			}else{
				echo $guild_roster->error;
			}
		}
	}
}
